use std::path::PathBuf;

use clap::Parser;

#[derive(clap::Parser, Clone)]
struct Args {
    #[arg(long)]
    root_dir: String,

    #[arg(long)]
    pattern: Option<String>,

    #[arg(long)]
    projects_file: String,
}

fn main() -> eyre::Result<()> {
    let args = Args::parse();

    // Choose dir
    let root_dir = args.root_dir;

    let mut projects: Vec<PathBuf> = vec![];

    let pattern = regex::Regex::new(match args.pattern {
        Some(p) => Box::leak(p.into_boxed_str()),
        None => ".*\\.git$",
    })?;

    // Look for dir .git if found add path to map
    for entry in walkdir::WalkDir::new(root_dir)
        .follow_links(true)
        .into_iter()
        .filter_map(|e| e.ok())
    {
        let metadata = entry.metadata()?;
        if metadata.is_dir() {
            let name = entry.file_name().to_string_lossy();
            if pattern.is_match(&name) {
                projects.push(entry.into_path())
            }
        }
    }

    // Print
    println!("writing projects");
    let projects_str = projects
        .into_iter()
        .map(|mut p| {
            p.pop();
            p
        })
        .map(|p| p.to_string_lossy().to_string())
        .collect::<Vec<String>>();

    let content = projects_str.join("\n");
    std::fs::write(args.projects_file, content)?;

    Ok(())
}
